/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ibm.jzos.PdsDirectory;
import com.ibm.jzos.ZFile;

/**
 * A Utility class with static methods for accessing a PDS library
 * or directory containing ADATA files.
 */
public class LibraryUtil {

	/**
	 * Return a list of adatafile names which are members/files in the
	 * PDS/directory name given
	 */
	public static List getLibraryMembers(String adatafile) throws IOException {
		if (adatafile.startsWith("//")) {
			return getPDSDatasetAndMembers(adatafile);
		} else {
			return getDirectoryFiles(adatafile);
		}
	}
	
	/**
	 * Helper method to resolve a pathname starting with tilde (~), 
	 * referring to the home directory
	 */
	public static String resolveLeadingTilde(String path) {
		if (path == null || !path.startsWith("~")) return path;
		return System.getProperty("user.home", "") + path.substring(1);
	}
	
	/**
	 * Make a PDS member name from a //Datasetname or //DD:DDNAME
	 */
	private static String getDatasetAndMemberName(String dsname, String member) {
		if (dsname.toUpperCase().indexOf("DD:") >= 0) {
			return dsname + "(" + member + ")";
		} else {
			dsname = ZFile.getFullyQualifiedDSN(dsname);
			return "//'" + dsname + "(" + member + ")'";
		}
	}

	/**
	 * Return a list of filenames in the given directory name,
	 * or a list containing the filename itself it is not a directory.
	 */
	private static List getDirectoryFiles(String dirname) throws IOException {
		ArrayList answer = new ArrayList();
		File d = new File(dirname);
		if (!d.exists()) {
			throw new IllegalArgumentException("The file or directory \"" + dirname + "\" does not exist");
		}
		if (d.isDirectory()) {
			File[] files = d.listFiles();
			for (int i=0; i<files.length; i++) {
				File f = files[i];
				if (f.isFile()) answer.add(f.getCanonicalPath());
			}
		} else {
			answer.add(dirname);
		}
		return answer;
	}

	/**
	 * Return a list of members in the given PDS name,
	 * or a list containing the dataset itself it is not a PDS.
	 */
	private static List getPDSDatasetAndMembers(String pdsname) throws IOException {
		
		ArrayList answer = new ArrayList();
		PdsDirectory pdsDir = null;
		try {
			pdsDir = new PdsDirectory(pdsname);
		} catch (IOException ioe) {
			// Probably not a PDS or we can't open it.
			// Just return its name
			answer.add(pdsname);
			return answer;
		}
		// we now have an open PDS directory
		try {
			for (Iterator i=pdsDir.iterator(); i.hasNext(); ) {
				PdsDirectory.MemberInfo mem = (PdsDirectory.MemberInfo)i.next();
				answer.add(getDatasetAndMemberName(pdsname, mem.getName()));
			}
		} finally {
			try { pdsDir.close(); } catch(Throwable ignore) {}
		}
		return answer;
	}

}
