/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen;

import java.io.IOException;
import java.io.Writer;

/**
 * A class to write lines of Java source code.
 * Automatically handles indenting and provides a C printf() style interface.
 */
public class LineFormatter {

	private String indent = "";
	private boolean lineInProgress = false;
	/** The Writer instance wrapped by this formatter */
	private Writer writer;
	
	/**
	 * Construct an instance on a Writer
	 * @param writer
	 */
	public LineFormatter(Writer writer) {
		this.writer = writer;
	}
	
	/**
	 * Gets the current line indent characters.
	 * @return String
	 */
	public String getIndent() {
		return indent;
	}
	
	/**
	 * Prints a string
	 */	
	public void print(String string) throws IOException {
		print(string, null);
	}
	
	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */
	public void print(String template, Object arg1) throws IOException {
		print(template, new Object[] {arg1});
	}
	
	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */	
	public void print(String template, Object arg1, Object arg2) throws IOException {
		print(template, new Object[] {arg1, arg2});
	}
	
	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */	
	public void print(String template, Object arg1, Object arg2, Object arg3) throws IOException {
		print(template, new Object[] {arg1, arg2, arg3});
	}
	
	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */	
	public void print(String template, Object arg1, Object arg2, Object arg3, Object arg4) throws IOException {
		print(template, new Object[] {arg1, arg2, arg3, arg4});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */	
	public void print(String template, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5) throws IOException {
		print(template, new Object[] {arg1, arg2, arg3, arg5});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */
	public void print(String template, Object[] args) throws IOException  {
		if (!lineInProgress) {
			writer.write(indent);
			lineInProgress = true;
		}
		int p = 0;
		int q = 0;
		int i = 0;
		if (args == null) {
			writer.write(template);
			return;
		}
		while (p < template.length() && (q = template.indexOf('%', p)) >= 0) {
			writer.write(template.substring(p,q));
			if (i < args.length && args[i] != null) {
				writer.write(args[i++].toString());
			}
			p = q+1;
		}
		if (p < template.length()) {
			writer.write(template.substring(p));
		}
	}

	/**
	 * Prints a new line
	 */
	public void println() throws IOException {
		writer.write("\n");
		lineInProgress = false;
	}

	/**
	 * Prints a string and new line
	 */
	public void println(String string) throws IOException {
		println(string, null);
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1) throws IOException {
		println(template, new Object[] {arg1});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1, Object arg2) throws IOException {
		println(template, new Object[] {arg1, arg2});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1, Object arg2, Object arg3) throws IOException {
		println(template, new Object[] {arg1, arg2, arg3});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1, Object arg2, Object arg3, Object arg4) throws IOException {
		println(template, new Object[] {arg1, arg2, arg3, arg4});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5) throws IOException {
		println(template, new Object[] {arg1, arg2, arg3, arg4, arg5});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%' and a trailing newline
	 */	
	public void println(String template, Object arg1, Object arg2, Object arg3, Object arg4, Object arg5, Object arg6) throws IOException {
		println(template, new Object[] {arg1, arg2, arg3, arg4, arg5, arg6});
	}

	/**
	 * Prints a template, imbedding arguments at positions marked by '%'
	 */
	public void println(String template, Object[] args) throws IOException {
		char last = template.charAt(template.length()-1);
		if (last == '}' && indent.length() > 0) {
			indent = indent.substring(0, indent.length() - 1);
		}
		print(template, args);
		writer.write("\n");
		lineInProgress = false;
		if (last == '{') {
			indent = indent + "\t";
		} 
	}

	/**
	 * Manually set the indent.
	 * This is normally not done, since the indent is automatically
	 * managed by the formatter.
	 * @param value
	 */
	public void setIndent(String value) {
		indent = value;
	}

}
