/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.asm;

import java.io.IOException;
import java.io.Writer;

import com.ibm.jzos.fields.BinaryAsIntField;
import com.ibm.jzos.fields.StringField;
import com.ibm.jzos.recordgen.ADataRecord;

/**
 * Describes a AData SOURCE record
 * <p>
 * @see ADataRecord
 */
public class ADataSourceRecord extends ADataRecord {
	
	/** cached stmnt number */
	private Integer _stmt;
	
	/** next continuation source line */
	private ADataSourceRecord _continuationRecord;
	
	/** related ADataDCDSRecord */
	private ADataDCDSRecord _dcdsRecord;
	
	/** related ADataSymbolRecord */
	private ADataSymbolRecord _symbolRecord;
	
	public static final int HEADER_LEN = 0x008C; 
	
	static { factory.setOffset(ADataRecord.ADATA_EQU_LEN); }  // TODO hmmmm

	// ADATA_RECSOURCE     EQU   X'0030'  - Source                            
	public static final int ADATA_RECSOURCE = 0x0030;
	
	// ADATA_EDNSOURCE     EQU   X'01'    - Source Edition                @v4c
	public static final int ADATA_EDNSOURCE = 0x01;
	
	// ADSRC_ESDID         DS    FL4        ESDID for the source record   @v4c
	protected static BinaryAsIntField _ADSRC_ESDID = (BinaryAsIntField)factory.getBinaryField(4,true);
	
	// ADSRC_STMT          DS    FL4        Statement number              @v4c
	protected static BinaryAsIntField _ADSRC_STMT = (BinaryAsIntField)factory.getBinaryField(4,true);

	// ADSRC_INPUT_REC     DS    FL4        Input source record number        
	protected static BinaryAsIntField _ADSRC_INPUT_REC = (BinaryAsIntField)factory.getBinaryField(4,true);
	
	// ADSRC_PARENT_REC    DS    FL4        Input source parent record number 
	protected static BinaryAsIntField _ADSRC_PARENT_REC = (BinaryAsIntField)factory.getBinaryField(4,true);

	// ADSRC_INPUT_NUM     DS    FL4        Input relative file number        
	protected static BinaryAsIntField _ADSRC_INPUT_NUM = (BinaryAsIntField)factory.getBinaryField(4,true);
	
	// ADSRC_PARENT_NUM    DS    FL4        Parent relative file number       
	protected static BinaryAsIntField _ADSRC_PARENT_NUM = (BinaryAsIntField)factory.getBinaryField(4,true);

	// ADSRC_LOCTR         DS    FL4        Source record location counter
	protected static BinaryAsIntField _ADSRC_LOCTR = (BinaryAsIntField)factory.getBinaryField(4,true);
	
	// ADSRC_REC_ORIGIN    DS    XL1        Where the source line comes from  
	protected static BinaryAsIntField _ADSRC_REC_ORIGIN = (BinaryAsIntField)factory.getBinaryField(1,false);

	// ADSRC_REC_PINP      EQU   X'01'      Source line from Primary Input
	public static final int ADSRC_REC_PINP = 0x01;
	
	// ADSRC_REC_MACGEN    EQU   X'02'      Source line from Macro generation
	public static final int ADSRC_REC_MACGEN = 0x02;

	// ADSRC_REC_MAC       EQU   X'03'      Source line from macro / copy code
	public static final int ADSRC_REC_MAC = 0x03;
	
	// *                   EQU   X'04'      Reserved                     1@L1D
	// ADSRC_REC_AINSERT   EQU   X'05'      Source line from AINSERT buf  @L1A
	public static final int ADSRC_REC_AINSERT = 0x05;
	
	// ADSRC_PARENT_ORIGIN DS    XL1        Where the parent line comes from  
    //                                      (Values as above)                 
	protected static BinaryAsIntField _ADSRC_PARENT_ORIGIN = (BinaryAsIntField)factory.getBinaryField(1,false);

	//*                                                                  @v4a
	// ADSRC_PRINT_GEN       EQU  *,X'80'   PRINT GEN                     @v4c
	public static final int ADSRC_PRINT_GEN = 0x80;

	// ADSRC_PRINT_DATA      EQU  *,X'40'   PRINT DATA                    @v4c
	public static final int ADSRC_PRINT_DATA = 0x40;

	// ADSRC_PRINT_ON        EQU  *,X'20'   PRINT ON                      @v4c
	public static final int ADSRC_PRINT_ON = 0x20;

	// ADSRC_PRINT_NOMSOURCE EQU  *,X'10'   PRINT NOMSOURCE (0 = MSOURCE) @v4c
	public static final int ADSRC_PRINT_NOMSOURCE = 0x10;

	// ADSRC_PRINT_UHEAD     EQU  *,X'08'   PRINT UHEAD                   @v4c
	public static final int ADSRC_PRINT_UHEAD = 0x08;

	// ADSRC_PRINT_MCALL     EQU  *,X'04'   PRINT MCALL                   @v4c
	public static final int ADSRC_PRINT_MCALL = 0x04;

	// ADSRC_PRINT_FLAGS     DS   XL1       Current print status flags    @v4c
	protected static BinaryAsIntField _ADSRC_PRINT_FLAGS = (BinaryAsIntField)factory.getBinaryField(1,false);
	
	// *                                                                  @v4a
	//                     DS    XL2        Reserved                      @l1c
	static { factory.getBinaryField(2, false); }
	
	// ADSRC_REC_TYPE      DS    XL1        Source record type                
	protected static BinaryAsIntField _ADSRC_REC_TYPE = (BinaryAsIntField)factory.getBinaryField(1,false);

	// ADSRC_REC_COMM      EQU   X'01'      Comment                           
	public static final int ADSRC_REC_COMM = 0x01;

	// ADSRC_REC_MACH      EQU   X'02'      Machine instruction               
	public static final int ADSRC_REC_MACH = 0x02;

	// ADSRC_REC_ASM       EQU   X'03'      Assembler instruction             
	public static final int ADSRC_REC_ASM = 0x03;

	// ADSRC_REC_MACR      EQU   X'04'      Macro instruction                 
	public static final int ADSRC_REC_MACR = 0x04;

	// ADSRC_REC_MACD      EQU   X'05'      Macro definition                  
	public static final int ADSRC_REC_MACD = 0x05;

	// ADSRC_ASM_OPCODE    DS    XL1        Assembler opcode   
	protected static BinaryAsIntField _ADSRC_ASM_OPCODE = (BinaryAsIntField)factory.getBinaryField(1,false);
	
	static { factory.setOffset(0x0078); }  // skipped over a bunch of fields that we don't current need
	
	// ADSRC_RECORD_OFF    DS    FL4        Offset of source record  
	protected static BinaryAsIntField _ADSRC_RECORD_OFF = (BinaryAsIntField)factory.getBinaryField(4,true);
	
	// ADSRC_RECORD_LEN    DS    FL4        Length of source record  
	protected static BinaryAsIntField _ADSRC_RECORD_LEN = (BinaryAsIntField)factory.getBinaryField(4,true);

	//                     DS    XL8        Reserved
	static { factory.getByteArrayField(8); }
	
	// ADSRC_EQU_LEN       EQU   *-ASMADATA                          
	public static final int ADSRC_EQU_LEN = factory.getOffset();

	// ADSRC_MEMBER_NAME   DS    0CL256
	static { factory.pushOffset(); }
	protected static StringField _ADSRC_MEMBER_NAME = factory.getStringField(256,true);
	static { factory.popOffset(); }

	// ADSRC_PARENT_NAME   DS    0CL256
	static { factory.pushOffset(); }
	protected static StringField _ADSRC_PARENT_NAME = factory.getStringField(256,true);
	static { factory.popOffset(); }

	// ADSRC_RECORD        DS    0CL80 
	static { factory.pushOffset(); }
	protected static StringField _ADSRC_RECORD = factory.getStringField(80,true);
	static { factory.popOffset(); }

	
	/**
	 * Construct an instance on an existing byte array.
	 * @param buffer the byte array used as backing memory for the structure
	 */
	public ADataSourceRecord(byte[] buffer) {
		super(buffer);
		if (buffer.length < ADSRC_EQU_LEN) {
			throw new IllegalArgumentException("buffer length");
		}
		bytes = buffer;
	}
	
	/**
	 * Answer the edition number that this (subtype instance) supports.
	 */
	public int getSupportedEdition() {
		return 1;  
	}	
	

	public int getSTMT() {
		if (_stmt == null) {
			_stmt = new Integer(_ADSRC_STMT.getInt(bytes));
		}
		return _stmt.intValue();
	}
	
	public int getESDID() {
		return _ADSRC_ESDID.getInt(bytes);
	}
	
	public int getRECORD_OFF() {
		return _ADSRC_RECORD_OFF.getInt(bytes);
	}
	
	public int getRECORD_LEN() {
		return _ADSRC_RECORD_LEN.getInt(bytes);
	}
	
	/**
	 * Answer a String containing the original source line
	 * @return String
	 */
	public String getSourceLine() {
		int offset = getRECORD_OFF();
		int len = getRECORD_LEN();
		StringField f = new StringField(offset, len);
		return f.getString(bytes);
	}
	
	/**
	 * Answer the associated DCDS record (may be null)
	 * @return ADataDCDSRecord
	 */
	public ADataDCDSRecord getDCDSRecord() {
		return _dcdsRecord;
	}

	/**
	 * Sets an associated DCDS record
	 * @param dcdsRecord ADataDCDSRecord
	 */
	public void setDCDSRecord(ADataDCDSRecord dcdsRecord) {
		_dcdsRecord = dcdsRecord;
	}

	/**
	 * Answer an associated ADataSymbolRecord (may be null)
	 * @return ADdataSymbolRecord
	 */
	public ADataSymbolRecord getSymbolRecord() {
		return _symbolRecord;
	}
	
	/**
	 * Sets an associated ADataSymbolRecord
	 * @param symbolRecord
	 */
	public void setSymbolRecord(ADataSymbolRecord symbolRecord) {
		_symbolRecord = symbolRecord;
	}

	/**
	 * Answer the (next) continuation ADataSourceRecord.  May be null.
	 * @return ADataSourceRecord
	 */
	public ADataSourceRecord getContinuationRecord() {
		return _continuationRecord;
	}

	/** 
	 * Set the next continuation ADataSourceRecord.
	 */ 
	public void setContinuationRecord(ADataSourceRecord record) {
		_continuationRecord = record;
	}

	/**
	 * Write a debug dump of the receiver.
	 */
	public void dumpOn(Writer writer) throws IOException {
		//super.dumpOn(writer);
		writer.write("" + getSTMT());
		writer.write(": "); 
		writer.write(getSourceLine());
		writer.write("\n");
	}
}
