/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.asm;

import java.io.IOException;
import java.io.Writer;

import com.ibm.jzos.ByteUtil;
import com.ibm.jzos.fields.BinaryAsIntField;
import com.ibm.jzos.fields.BinaryAsLongField;
import com.ibm.jzos.fields.StringField;
import com.ibm.jzos.recordgen.ADataRecord;

/**
 * Maps an AData Symbol record.
 * <p>
 * @see ADataRecord
 */
public class ADataSymbolRecord extends ADataRecord {
	
	public static final int FIXED_LEN = 0x003E; 
	
	static { factory.setOffset(ADataRecord.ADATA_EQU_LEN); }  // TODO hmmmm

	// ADATA_RECSYM        EQU   X'0042'  - Symbol
	public static final int ADATA_RECSYM = 0x0042;
	
	// ADATA_EDNSYM        EQU   X'01'    - Symbol Edition
	public static final int ADATA_EDNSYM = 0x01;
	
	// ADSYM_ESDID         DS    FL4        ESDID of the section in which
	//                                       the symbol is defined.
	protected static final BinaryAsIntField _ADSYM_ESDID = (BinaryAsIntField)factory.getBinaryField(4, true);
	
	// ADSYM_STMT          DS    FL4        Statement number
	protected static final BinaryAsIntField _ADSYM_STMT = (BinaryAsIntField)factory.getBinaryField(4, true);
	
	// ADSYM_LOCTR         DS    FL4        Location counter             
	protected static final BinaryAsIntField _ADSYM_LOCTR = (BinaryAsIntField)factory.getBinaryField(4, true);

	// ADSYM_TYPE          DS    XL1        Symbol section type
	protected static final BinaryAsIntField _ADSYM_TYPE = (BinaryAsIntField)factory.getBinaryField(1, false);
	
	// ADSYM_TYPE_UNDEF    EQU   X'00'      Undefined name
	public static int ADSYM_TYPE_UNDEF = 0x00;
	
	// ADSYM_TYPE_SECTION  EQU   X'01'      CSECT / RSECT name
	public static int ADSYM_TYPE_SECTION = 0x01;

	// ADSYM_TYPE_DSECT    EQU   X'02'      DSECT name
	public static int ADSYM_TYPE_DSECT = 0x02;

	// ADSYM_TYPE_COMMON   EQU   X'03'      Common section name          
	public static int ADSYM_TYPE_COMMON = 0x03;

	// ADSYM_TYPE_DXD      EQU   X'04'      Dummy External DSECT name    
	public static int ADSYM_TYPE_DXD = 0x04;

	// ADSYM_TYPE_VCON     EQU   X'05'      V-type constant name         
	public static int ADSYM_TYPE_VCON = 0x05;

	// ADSYM_TYPE_QUAL     EQU   X'06'      Qualifier                    
	public static int ADSYM_TYPE_QUAL = 0x06;

	// ADSYM_TYPE_XTRN     EQU   X'07'      EXTRN/WXTRN name             
	public static int ADSYM_TYPE_XTRN = 0x07;

	// ADSYM_TYPE_LOCTR    EQU   X'08'      LOCTR name                   
	public static int ADSYM_TYPE_LOCTR = 0x08;

	// ADSYM_TYPE_DUP      EQU   X'09'      Duplicate name               
	public static int ADSYM_TYPE_DUP = 0x09;

	// ADSYM_TYPE_LITERAL  EQU   X'0A'      Literal name                 
	public static int ADSYM_TYPE_LITERAL = 0x0A;

	// ADSYM_TYPE_ASTLIT   EQU   X'0B'      *-in-literal name            
	public static int ADSYM_TYPE_ASTLIT = 0x0B;

	// ADSYM_TYPE_EQU      EQU   X'0C'      EQU name                     
	public static int ADSYM_TYPE_EQU = 0x0C;

	// ADSYM_TYPE_ORDINARY EQU   X'0D'      Ordinary label               
	public static int ADSYM_TYPE_ORDINARY = 0x0D;

	// ADSYM_TYPE_UNRES    EQU   X'0E'      Unresolvable EQU, DC or DS   
	public static int ADSYM_TYPE_UNRES = 0x0E;

	// ADSYM_DUP           DS    FL4        Duplication factor 
	protected static final BinaryAsIntField _ADSYM_DUP = (BinaryAsIntField)factory.getBinaryField(4, true);
	
	// ADSYM_ATTR          DS    XL1        Type attribyte                
	protected static final StringField _ADSYM_ATTR = factory.getStringField(1, false);

	// ADSYM_ASM_TYPE      DS    CL4        Assembler Type
	protected static final StringField _ADSYM_ASM_TYPE = factory.getStringField(4, true);
	
	// ADSYM_PROGRAM_TYPE  DS    XL4        Program Type                  
	protected static final BinaryAsLongField _ADSYM_PROGRAM_TYPE = (BinaryAsLongField)factory.getBinaryField(4, false);

	// ADSYM_BYTE_LEN      DS    FL4        Length in bytes               
	protected static final BinaryAsIntField _ADSYM_BYTE_LEN = (BinaryAsIntField)factory.getBinaryField(4, true);

	// ADSYM_INT           DS    HL2        Number of positions occupied by
	//                                       int portion of fixed-point.
	protected static final BinaryAsIntField _ADSYM_INT = (BinaryAsIntField)factory.getBinaryField(2, true);
	
	// ADSYM_SCALE         DS    HL2        Number of positions occupied by
	//                                       fract portion of fixed-point.
	protected static final BinaryAsIntField _ADSYM_SCALE = (BinaryAsIntField)factory.getBinaryField(2, true);
	// *                                                                  
	// ADSYM_RELOC         EQU   *,X'80'    Operand points to a relocatable
	//                                       expression.
	public static final int ADSYM_RELOC = 0x80;
	
	// ADSYM_COMPLEX       EQU   *,X'C0'    Operand points to a complex   
	//                                       relocatable symbol.          
	public static final int ADSYM_COMPLEX = 0xC0;

	// ADSYM_RELOC_FLAG    DS    XL1        Relocatable flag              
	protected static final BinaryAsIntField _ADSYM_RELOC_FLAG = (BinaryAsIntField)factory.getBinaryField(1, false);

	// *                                                                  
	//                     DS    XL7        Reserved
	static { factory.getByteArrayField(7); }
	
	// ADSYM_NAME_OFF      DS    FL4        Offset of Symbol Name         
	protected static final BinaryAsIntField _ADSYM_NAME_OFF = (BinaryAsIntField)factory.getBinaryField(4, true);

	// ADSYM_NAME_LEN      DS    FL4        Length of Symbol Name         
	protected static final BinaryAsIntField _ADSYM_NAME_LEN = (BinaryAsIntField)factory.getBinaryField(4, true);

	// ADSYM_EQU_LEN       EQU   *-ASMADATA
	public static final int ADSYM_EQU_LEN = factory.getOffset();
	
	// *                                                                  
	// *  This is now the end of the fixed portion of the record.         
	// *                                                                  
	// ADSYM_NAME          DS    0CL63      Symbol Name (variable name)   
	static { factory.pushOffset(); }
	protected static final StringField _ADSYM_NAME = factory.getStringField(63, true);
	static { factory.popOffset(); }
	
	/**
	 * Construct an instance on an existing byte array.
	 * @param buffer the byte array used as backing memory for the structure
	 */
	public ADataSymbolRecord(byte[] buffer) {
		super(buffer);
		if (buffer.length < ADSYM_EQU_LEN) {
			throw new IllegalArgumentException("buffer length");
		}
		bytes = buffer;
	}
	
	/**
	 * Answer the edition number that this (subtype instance) supports.
	 */
	public int getSupportedEdition() {
		return 1;  
	}		

	public int getESDID() {
		return _ADSYM_ESDID.getInt(bytes);
	}
	
	public int getSTMT() {
		return _ADSYM_STMT.getInt(bytes);
	}
	
	public int getLOCTR() {
		return _ADSYM_LOCTR.getInt(bytes);
	}
	
	public int getTYPE() {
		return _ADSYM_TYPE.getInt(bytes);
	}
	
	public int getDUP() {
		return _ADSYM_DUP.getInt(bytes);
	}
	
	public String getATTR() {
		return _ADSYM_ATTR.getString(bytes);
	}
	
	public String getASM_TYPE() {
		return _ADSYM_ASM_TYPE.getString(bytes);
	}
	
	public long getPROGRAM_TYPE() {
		return _ADSYM_PROGRAM_TYPE.getLong(bytes);
	}

	public int getBYTE_LEN() {
		return _ADSYM_BYTE_LEN.getInt(bytes);
	}

	public int getINT() {
		return _ADSYM_INT.getInt(bytes);
	}

	public int getSCALE() {
		return _ADSYM_SCALE.getInt(bytes);
	}

	public int getRELOC_FLAG() {
		return _ADSYM_RELOC_FLAG.getInt(bytes);
	}

	public int getNAME_OFF() {
		return _ADSYM_NAME_OFF.getInt(bytes);
	}

	public int getNAME_LEN() {
		return _ADSYM_NAME_LEN.getInt(bytes);
	}

	/**
	 * Answer a String containing the symbol (label) name
	 */
	public String getSymbolName() {
		int offset = getNAME_OFF();
		int len = getNAME_LEN();
		StringField f = new StringField(offset, len);
		return f.getString(bytes);
	}
	
	/**
	 * Write debugging information for the receiver.
	 */
	public void dumpOn(Writer writer) throws IOException {
		// super.dumpOn(writer);
		writer.write("Symbol: ");
		writer.write(getSymbolName());
		writer.write("  ESDID=");
		writer.write(Integer.toString(getESDID()));
		writer.write("  STMT=");
		writer.write(Integer.toString(getSTMT()));
		writer.write("  LOCTR=0x");
		writer.write(ByteUtil.toHexString(getLOCTR(), 4));
		writer.write("  TYPE=0x");
		writer.write(ByteUtil.toHexString(getTYPE(), 2));
		writer.write("  ATTR=");
		writer.write(getATTR());
		writer.write("\n");
		writer.write("    ASM_TYPE=");
		writer.write(getASM_TYPE());
		writer.write("  DUP=");
		writer.write(Integer.toString(getDUP()));
		writer.write("  BYTE_LEN=");
		writer.write(Integer.toString(getBYTE_LEN()));
		writer.write("  INT=");
		writer.write(Integer.toString(getINT()));
		writer.write("  SCALE=");
		writer.write(Integer.toString(getSCALE()));
		writer.write("  RELOC_FLAG=0x");
		writer.write(ByteUtil.toHexString(getRELOC_FLAG(),2));
		writer.write("\n");
	}
}
