/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.asm;

import java.io.IOException;
import java.io.Writer;

/**
 * A subtype of ElementSpec, part of a RecordSpec, used to describe
 * an assembler DSECT constant.
 * <p>
 * @see RecordSpec
 * @see ElementSpec
 */
public class ConstantSpec extends ElementSpec {

	private boolean relocatable;
	private Object value;
	private String typeAttr;

	
	/**
	 * Answer true if the receiver is from a relocatable symbol
	 * @return boolean
	 */
	public boolean isRelocatable() {
		return relocatable;
	}
	
	/**
	 * @see #isRelocatable
	 */
	public void setRelocatable(boolean relocatable) {
		this.relocatable = relocatable;
	}
	
	/**
	 * Answer a Character or Integer object value for this constant
	 * @return Object a Character or Integer object wrapper
	 */
	public Object getValue() {
		return value;
	}
	
	/**
	 * @see #getValue()
	 */
	public void setValue(Object value) {
		this.value = value;
	}

	/**
	 * @return String the one character type attribute
	 */
	public String getTypeAttr() {
		return typeAttr;
	}

	/**
	 * @see #getTypeAttr()
	 */
	public void setTypeAttr(String typeAttr) {
		this.typeAttr = typeAttr;
	}

	
	/**
	 * Write a debug dump of the receiver.
	 */
	public void dumpOn(Writer writer) throws IOException {
		
		super.dumpOn(writer);
		writer.write("  relocatable=");
		writer.write(isRelocatable() ? "true" : "false");
		writer.write("  value=(");
		writer.write(getUnqualifiedClassname(value.getClass()));
		writer.write(") ");
		writer.write(value.toString());
		writer.write("\n");
	}
}
