/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import java.io.IOException;
import java.io.Writer;

import com.ibm.jzos.fields.AssemblerDatatypeFactory;
import com.ibm.jzos.fields.BinaryAsIntField;
import com.ibm.jzos.fields.StringField;
import com.ibm.jzos.recordgen.ADataRecord;

/**
 * Class to map SYSADATA Sources records emitted by the z/OS Enterprise COBOL Compiler
 * Generated by com.ibm.jzos.recordgen.asm.RecordClassGenerator on: Mon Aug 13 17:44:29 EDT 2007
 */
public class ADataSourceRecord extends ADataRecord {

	protected static AssemblerDatatypeFactory factory = new AssemblerDatatypeFactory();
	static { factory.setOffset(ADataRecord.ADATA_EQU_LEN); } 	

	/** ADSRC               DSECT */
	public static int ADSRC = factory.getOffset();
	
	/** ADATA_RECSOURCE     EQU   X'0038'  - Source */
	public static int ADATA_RECSOURCE = 0x38;

	/** ADATA_EDNSOURCE     EQU   X'01'    - Source Edition */
	public static int ADATA_EDNSOURCE = 0x1;

	/** LINE_NUM            DS    F     Listing line number of the source rec */
	protected static BinaryAsIntField LINE_NUM = factory.getBinaryAsIntField(4, true);

	/** INPUT_REC_NUM       DS    F     Record number in current input file */
	protected static BinaryAsIntField INPUT_REC_NUM = factory.getBinaryAsIntField(4, true);

	/** PRIM_FILE_NUM       DS    H     The input file's assigned sequence num x <br/>
	                                    if this is from the primary input file */
	protected static BinaryAsIntField PRIM_FILE_NUM = factory.getBinaryAsIntField(2, true);

	/** LIB_FILE_NUM        DS    H     Library file's assigned sequence num */
	protected static BinaryAsIntField LIB_FILE_NUM = factory.getBinaryAsIntField(2, true);

	/**                     DS    CL8   Reserved for future use */
	static { factory.getStringField(8, true); }

	/** PARENT_REC_NUM       DS    F     The parent source record number */
	protected static BinaryAsIntField PARENT_REC_NUM = factory.getBinaryAsIntField(4, true);

	/** PARENT_PRIM_FILE_NUM DS   H     The parent file's assigned seq num */
	protected static BinaryAsIntField PARENT_PRIM_FILE_NUM = factory.getBinaryAsIntField(2, true);

	/** PARENT_LIB_FILE_NUM  DS   H     The parent library file's assigned seq */
	protected static BinaryAsIntField PARENT_LIB_FILE_NUM = factory.getBinaryAsIntField(2, true);

	/**                     DS    CL8   Reserved for future use */
	static { factory.getStringField(8, true); }

	/** RECORD_LEN          DS    H     The length of the record following       */
	protected static BinaryAsIntField RECORD_LEN = factory.getBinaryAsIntField(2, true);

	/**                     DS    CL10  Reserved for future use */
	static { factory.getStringField(10, true); }

	/** SOURCE_RECORD       DS    C     Length varies by RECORD_LEN */
	protected static StringField  SOURCE_RECORD = factory.getStringField(1, false);

	// Instance variables used to cache field values 
	private Integer lineNum;
	private Integer inputRecNum;
	private Integer primFileNum;
	private Integer libFileNum;
	private Integer parentRecNum;
	private Integer parentPrimFileNum;
	private Integer parentLibFileNum;
	private Integer recordLen;
	private String  sourceRecord;


	/**
	 * Construct an instance on an existing byte array.
	 * @param buffer the byte array used as backing memory for the structure
	 */
	public ADataSourceRecord(byte[] buffer) {
		super(buffer);
		if (buffer.length < SOURCE_RECORD.getOffset()) {
			throw new IllegalArgumentException("buffer length");
		}
	}	

	public int getLineNum() {
		if (lineNum == null) {
			lineNum = new Integer(LINE_NUM.getInt(bytes));
		}
		return lineNum.intValue();
	}

	public void setLineNum(int lineNum) {
		if (LINE_NUM.equals(this.lineNum, lineNum))
			return;
		LINE_NUM.putInt(lineNum, bytes);
		this.lineNum = new Integer(lineNum);
	}

	public int getInputRecNum() {
		if (inputRecNum == null) {
			inputRecNum = new Integer(INPUT_REC_NUM.getInt(bytes));
		}
		return inputRecNum.intValue();
	}

	public void setInputRecNum(int inputRecNum) {
		if (INPUT_REC_NUM.equals(this.inputRecNum, inputRecNum))
			return;
		INPUT_REC_NUM.putInt(inputRecNum, bytes);
		this.inputRecNum = new Integer(inputRecNum);
	}

	public int getPrimFileNum() {
		if (primFileNum == null) {
			primFileNum = new Integer(PRIM_FILE_NUM.getInt(bytes));
		}
		return primFileNum.intValue();
	}

	public void setPrimFileNum(int primFileNum) {
		if (PRIM_FILE_NUM.equals(this.primFileNum, primFileNum))
			return;
		PRIM_FILE_NUM.putInt(primFileNum, bytes);
		this.primFileNum = new Integer(primFileNum);
	}

	public int getLibFileNum() {
		if (libFileNum == null) {
			libFileNum = new Integer(LIB_FILE_NUM.getInt(bytes));
		}
		return libFileNum.intValue();
	}

	public void setLibFileNum(int libFileNum) {
		if (LIB_FILE_NUM.equals(this.libFileNum, libFileNum))
			return;
		LIB_FILE_NUM.putInt(libFileNum, bytes);
		this.libFileNum = new Integer(libFileNum);
	}

	public int getParentRecNum() {
		if (parentRecNum == null) {
			parentRecNum = new Integer(PARENT_REC_NUM.getInt(bytes));
		}
		return parentRecNum.intValue();
	}

	public void setParentRecUm(int parentRecNum) {
		if (PARENT_REC_NUM.equals(this.parentRecNum, parentRecNum))
			return;
		PARENT_REC_NUM.putInt(parentRecNum, bytes);
		this.parentRecNum = new Integer(parentRecNum);
	}

	public int getParentPrimFileNum() {
		if (parentPrimFileNum == null) {
			parentPrimFileNum = new Integer(PARENT_PRIM_FILE_NUM.getInt(bytes));
		}
		return parentPrimFileNum.intValue();
	}

	public void setParentPrimFileNum(int parentPrimFileNum) {
		if (PARENT_PRIM_FILE_NUM.equals(this.parentPrimFileNum, parentPrimFileNum))
			return;
		PARENT_PRIM_FILE_NUM.putInt(parentPrimFileNum, bytes);
		this.parentPrimFileNum = new Integer(parentPrimFileNum);
	}

	public int getParentLibFileNum() {
		if (parentLibFileNum == null) {
			parentLibFileNum = new Integer(PARENT_LIB_FILE_NUM.getInt(bytes));
		}
		return parentLibFileNum.intValue();
	}

	public void setParentLibFileNum(int parentLibFileNum) {
		if (PARENT_LIB_FILE_NUM.equals(this.parentLibFileNum, parentLibFileNum))
			return;
		PARENT_LIB_FILE_NUM.putInt(parentLibFileNum, bytes);
		this.parentLibFileNum = new Integer(parentLibFileNum);
	}

	public int getRecordLen() {
		if (recordLen == null) {
			recordLen = new Integer(RECORD_LEN.getInt(bytes));
		}
		return recordLen.intValue();
	}

	public void setRecordLen(int recordLen) {
		if (RECORD_LEN.equals(this.recordLen, recordLen))
			return;
		RECORD_LEN.putInt(recordLen, bytes);
		this.recordLen = new Integer(recordLen);
	}

	public String getSourceRecord() {
		if (this.sourceRecord == null) {
			SOURCE_RECORD.setByteLength(getRecordLen());  // field length varies
			this.sourceRecord = SOURCE_RECORD.getString(bytes);
		}
		return sourceRecord;
	}
	
	public void setSourceRecord(String sourceRecord) {
		if (SOURCE_RECORD.equals(this.sourceRecord, sourceRecord)) 
			return;
		SOURCE_RECORD.setByteLength(sourceRecord.length()); // field length varies
		SOURCE_RECORD.putString(sourceRecord, bytes);
	}
	
	/**
	 * Write a debug dump of the receiver.
	 */
	public void dumpOn(Writer writer) throws IOException {
		//super.dumpOn(writer);
		writer.write("" + getLineNum());
		writer.write(" inputRecNum=" + getInputRecNum());
		writer.write(" parentRecNum=" + getParentRecNum());
		writer.write(" : "); 
		writer.write(getSourceRecord());
		writer.write("\n");
	}	
}
