/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import java.io.IOException;
import java.io.Writer;

import com.ibm.jzos.ByteUtil;
import com.ibm.jzos.fields.AssemblerDatatypeFactory;
import com.ibm.jzos.fields.BinaryAsIntField;
import com.ibm.jzos.fields.StringField;
import com.ibm.jzos.recordgen.ADataRecord;

/**
 * Class to map SYSADATA Symbol records emitted by the z/OS Enterprise COBOL Compiler
 * Generated by com.ibm.jzos.recordgen.asm.RecordClassGenerator on: Mon Aug 13 17:44:29 EDT 2007
 */
public class ADataSymbolRecord extends ADataRecord {

	protected static AssemblerDatatypeFactory factory = new AssemblerDatatypeFactory();

	static { factory.setOffset(ADataRecord.ADATA_EQU_LEN); }
	
	/** ADSYM               DSECT */
	public static int ADSYM = factory.getOffset();

	/** ADATA_RECSYM       EQU   X'0042'  - Symbol */
	public static final int ADATA_RECSYM = 0x42;

	/** ADATA_EDNSOURCE     EQU   X'01'    - Source Edition */
	public static final int ADATA_EDNSOURCE = 0x1;

	/** SYMBOL_ID           DS    F     Unique Symbol ID */
	protected static BinaryAsIntField SYMBOL_ID = factory.getBinaryAsIntField(4, true);

	/** LINE_NUM            DS    F     Listing line number of the source rec */
	protected static BinaryAsIntField LINE_NUM = factory.getBinaryAsIntField(4, true);

	/** LEVEL               DS    XL1   True level-number of symbol */
	protected static BinaryAsIntField LEVEL = factory.getBinaryAsIntField(1, false);

	/** QUAL_IND            DS    XL1 */
	protected static BinaryAsIntField QUAL_IND = factory.getBinaryAsIntField(1, false);

	/** QUAL_UNIQUE         EQU   X'00' Unique name; no qualification needed */
	public static final int QUAL_UNIQUE = 0x0;

	/** QUAL_NOT_UNIQUE     EQU   X'01' Needs qualification */
	public static final int QUAL_NOT_UNIQUE = 0x1;

	/** SYMBOL_TYPE         DS    XL1 */
	protected static BinaryAsIntField SYMBOL_TYPE = factory.getBinaryAsIntField(1, false);

	/** SYMT_CLASS_NAME     EQU   X'68' Class-name (Class-ID) */
	public static final int SYMT_CLASS_NAME = 0x68;

	/** SYMT_METHOD_NAME    EQU   X'58' Method-name */
	public static final int SYMT_METHOD_NAME = 0x58;

	/** SYMT_DATA_NAME      EQU   X'40' Data-name */
	public static final int SYMT_DATA_NAME = 0x40;

	/** SYMT_DATA_PROC      EQU   X'20' Procedure-name */
	public static final int SYMT_DATA_PROC = 0x20;

	/** SYMT_MENONIC        EQU   X'10' Mnemonic-name */
	public static final int SYMT_MENONIC = 0x10;

	/** SYMT_PROG_NAME      EQU   X'08' Program-name */
	public static final int SYMT_PROG_NAME = 0x8;

	/** SYMT_RESERVED       EQU   X'81' Reserved */
	public static final int SYMT_RESERVED = 0x81;

	/** * The following fields are ORed into the above types; when applicable: <br/>
	    SYMBOL_EXTERNAL     EQU   X'04' External */
	public static final int SYMBOL_EXTERNAL = 0x4;

	/** SYMBOL_GLOBAL       EQU   X'02' Global */
	public static final int SYMBOL_GLOBAL = 0x2;

	/** SYMBOL_ATTR         DS    XL1 */
	protected static BinaryAsIntField SYMBOL_ATTR = factory.getBinaryAsIntField(1, false);

	/** SYMA_NUMERIC        EQU   X'01' Numeric */
	public static final int SYMA_NUMERIC = 0x1;

	/** SYMA_CHAR           EQU   X'02' Elementery character type */
	public static final int SYMA_CHAR = 0x2;

	/** SYMA_GROUP          EQU   X'03' Group */
	public static final int SYMA_GROUP = 0x3;

	/** SYMA_POINTER        EQU   X'04' Pointer */
	public static final int SYMA_POINTER = 0x4;

	/** SYMA_INDEX_ITEM     EQU   X'05' Index data item */
	public static final int SYMA_INDEX_ITEM = 0x5;

	/** SYMA_INDEX_NAME     EQU   X'06' Index-name */
	public static final int SYMA_INDEX_NAME = 0x6;

	/** SYMA_CONDITION      EQU   X'07' Condition */
	public static final int SYMA_CONDITION = 0x7;

	/** SYMA_FILE           EQU   X'0F' File */
	public static final int SYMA_FILE = 0xf;

	/** SYMA_SORT_FILE      EQU   X'10' Sort file */
	public static final int SYMA_SORT_FILE = 0x10;

	/** SYMA_CLASS_NAME     EQU   X'17' Class-name (repository) */
	public static final int SYMA_CLASS_NAME = 0x17;

	/** SYMA_OBJECT_REF     EQU   X'18' Object reference */
	public static final int SYMA_OBJECT_REF = 0x18;

	/** CLAUSES             DS    XL1   Clauses specified in the symbol def */
	protected static BinaryAsIntField CLAUSES = factory.getBinaryAsIntField(1, false);

	/** CLAUSE_VALUE        EQU   X'80' Value */
	public static final int CLAUSE_VALUE = 0x80;

	/** CLAUSE_INDEXED      EQU   X'40' Indexed */
	public static final int CLAUSE_INDEXED = 0x40;

	/** CLAUSE_REDEFINES    EQU   X'20' Redefines */
	public static final int CLAUSE_REDEFINES = 0x20;

	/** CLAUSE_RENAMES      EQU   X'10' Renames */
	public static final int CLAUSE_RENAMES = 0x10;

	/** CLAUSE_OCCURS       EQU   X'08' Occurs */
	public static final int CLAUSE_OCCURS = 0x8;

	/** CLAUSE_HAS_OCCURS_KEYS   EQU   X'04' Has Occurs keys */
	public static final int CLAUSE_HAS_OCCURS_KEYS = 0x4;

	/** CLAUSE_ODO          EQU   X'02' Occurs Depending On */
	public static final int CLAUSE_ODO = 0x2;

	/** CLAUSE_OCCURS_IN_PARENT  EQU   X'01' Occurs in parent */
	public static final int CLAUSE_OCCURS_IN_PARENT = 0x1;

	/**  <br/>
	    FLAGS1              DS    XL1 */
	protected static BinaryAsIntField FLAGS1 = factory.getBinaryAsIntField(1, false);

	/** FLG1_REDEFINED      EQU   X'80' */
	public static final int FLG1_REDEFINED = 0x80;

	/** FLG1_RENAMED        EQU   X'40' */
	public static final int FLG1_RENAMED = 0x40;

	/** FLG1_SYNCHRONIZED   EQU   X'20' */
	public static final int FLG1_SYNCHRONIZED = 0x20;

	/** FLG1_IMPL_REDEFINED EQU   X'10' */
	public static final int FLG1_IMPL_REDEFINED = 0x10;

	/** FLG1_DATE_FIELD     EQU   X'08' */
	public static int FLG1_DATE_FIELD = 0x8;

	/** FLG1_IMPL_REDEFINES EQU   X'04' */
	public static final int FLG1_IMPL_REDEFINES = 0x4;

	/** FLG1_FILLER         EQU   X'02' */
	public static final int FLG1_FILLER = 0x2;

	/** FLG1_LEVEL77        EQU   X'01' */
	public static final int FLG1_LEVEL77 = 0x1;

	/**  <br/>
	    FLAGS2              DS    XL1 */
	protected static BinaryAsIntField FLAGS2 = factory.getBinaryAsIntField(1, false);

	/** * For symbols that have an attribute of Numeric (x'01'): <br/>
	    FLG2_BINARY         EQU   X'80' */
	public static final int FLG2_BINARY = 0x80;

	/** FLG2_EXT_FLOAT      EQU   X'40' */
	public static final int FLG2_EXT_FLOAT = 0x40;

	/** FLG2_INT_FLOAT      EQU   X'20' */
	public static final int FLG2_INT_FLOAT = 0x20;

	/** FLG2_PACKED         EQU   X'10' */
	public static final int FLG2_PACKED = 0x10;

	/** FLG2_EXT_DECIMAL    EQU   X'08' */
	public static final int FLG2_EXT_DECIMAL = 0x8;

	/** FLG2_SCALED_NEG     EQU   X'04' */
	public static int FLG2_SCALED_NEG = 0x4;

	/** FLG2_NUMERIC_EDIT   EQU   X'02' */
	public static final int FLG2_NUMERIC_EDIT = 0x2;

	/** FLG2_RESERVED       EQU   X'01' */
	public static final int FLG2_RESERVED = 0x1;

	/** * For symbols that have an attribute of Elementary Char or Group: <br/>
	    FLG2_ALPHABETIC     EQU   X'80' */
	public static final int FLG2_ALPHABETIC = 0x80;

	/** FLG2_ALPHANUM       EQU   X'40' */
	public static final int FLG2_ALPHANUM = 0x40;

	/** FLG2_ALPHANUM_EDITED EQU  X'20' */
	public static final int FLG2_ALPHANUM_EDITED = 0x20;

	/** FLG2_GROUP_WITH_ODO EQU   X'10' */
	public static final int FLG2_GROUP_WITH_ODO = 0x10;

	/** FLG2_DBCS_ITEM      EQU   X'08' */
	public static final int FLG2_DBCS_ITEM = 0x8;

	/** FLG2_GROUP_VAR_LEN  EQU   X'04' */
	public static final int FLG2_GROUP_VAR_LEN = 0x4;

	/** FLG2_EGCS_ITEM      EQU   X'02' */
	public static final int FLG2_EGCS_ITEM = 0x2;

	/** FLG2_EGCS_EDITED    EQU   X'01' */
	public static final int FLG2_EGCS_EDITED = 0x1;

	/**  <br/>
	    FLAGS3              DS    XL1   For file types */
	protected static BinaryAsIntField FLAGS3 = factory.getBinaryAsIntField(1, false);

	/** FILE_ORG            DS    XL1 */
	protected static BinaryAsIntField FILE_ORG = factory.getBinaryAsIntField(1, false);

	/**  <br/>
	    USAGE_CLAUSE        DS    FL1 */
	protected static BinaryAsIntField USAGE_CLAUSE = factory.getBinaryAsIntField(1, true);

	/** USAGE_DISPLAY       EQU   X'00' */
	public static final int USAGE_DISPLAY = 0x0;

	/** USAGE_COMP1         EQU   X'01' */
	public static final int USAGE_COMP1 = 0x1;

	/** USAGE_COMP2         EQU   X'02' */
	public static final int USAGE_COMP2 = 0x2;

	/** USAGE_PACKED        EQU   X'03' AKA COMP-3 */
	public static final int USAGE_PACKED = 0x3;

	/** USAGE_BINARY        EQU   X'04' AKA COMP-4 */
	public static final int USAGE_BINARY = 0x4;

	/** USAGE_DISPLAY1      EQU   X'05' */
	public static final int USAGE_DISPLAY1 = 0x5;

	/** USAGE_POINTER       EQU   X'06' */
	public static final int USAGE_POINTER = 0x6;

	/** USAGE_INDEX         EQU   X'07' */
	public static final int USAGE_INDEX = 0x7;

	/** USAGE_PROC_PTR      EQU   X'08' */
	public static final int USAGE_PROC_PTR = 0x8;

	/** USAGE_OBJ_REF       EQU   X'09' */
	public static final int USAGE_OBJ_REF = 0x9;

	/** USAGE_NATIONAL      EQU   X'0B' */
	public static final int USAGE_NATIONAL = 0xb;

	/** USAGE_FUNC_PTR      EQU   X'0A' */
	public static final int USAGE_FUNC_PTR = 0xa;

	/**  <br/>
	    SIGN_CLAUSE         DS    FL1 */
	protected static BinaryAsIntField SIGN_CLAUSE = factory.getBinaryAsIntField(1, true);

	/** SIGN_NONE           EQU   X'00' */
	public static final int SIGN_NONE = 0x0;

	/** SIGN_LEADING        EQU   X'01' */
	public static final int SIGN_LEADING = 0x1;

	/** SIGN_LEADING_SEP    EQU   X'02' */
	public static final int SIGN_LEADING_SEP = 0x2;

	/** SIGN_TRAIL          EQU   X'03' */
	public static final int SIGN_TRAIL = 0x3;

	/** SIGN_TRAIL_SEP      EQU   X'04' */
	public static final int SIGN_TRAIL_SEP = 0x4;

	/**  <br/>
	    INDICATORS          DS    FL1 */
	protected static BinaryAsIntField INDICATORS = factory.getBinaryAsIntField(1, true);

	/** IND_JUSTIFIED       EQU   X'01' */
	public static final int IND_JUSTIFIED = 0x1;

	/** IND_BLANK_WHEN_ZERO EQU   X'02' */
	public static final int IND_BLANK_WHEN_ZERO = 0x2;

	/**  <br/>
	    SIZE                DS    F      Size in bytes */
	protected static BinaryAsIntField SIZE = factory.getBinaryAsIntField(4, true);

	/** PRECISION           DS    FL1    For fixed or floating point item */
	protected static BinaryAsIntField PRECISION = factory.getBinaryAsIntField(1, true);

	/** SCALE               DS    FL1    For fixed, number of digits right */
	protected static BinaryAsIntField SCALE = factory.getBinaryAsIntField(1, true);

	/**  <br/>
	    BASE_LOC_TYPE       DS    FL1 */
	protected static BinaryAsIntField BASE_LOC_TYPE = factory.getBinaryAsIntField(1, true);

	/** LOC_FILE            EQU   1 */
	public static final int LOC_FILE = 0x1;

	/** LOC_WORKING_STOR    EQU   2 */
	public static final int LOC_WORKING_STOR = 0x2;

	/** LOC_LINKAGE_SEC     EQU   3 */
	public static final int LOC_LINKAGE_SEC = 0x3;

	/** LOC_SPECIAL_REGS    EQU   5 */
	public static final int LOC_SPECIAL_REGS = 0x5;

	/** LOC_INDEXED         EQU   7 */
	public static final int LOC_INDEXED = 0x7;

	/** LOC_COMREG          EQU   9 */
	public static final int LOC_COMREG = 0x9;

	/** LOC_UPSI            EQU   10 */
	public static final int LOC_UPSI = 0xa;

	/** LOC_VARLOC_ITEMS    EQU   13 */
	public static final int LOC_VARLOC_ITEMS = 0xd;

	/** LOC_EXTERN_DATA     EQU   14 */
	public static final int LOC_EXTERN_DATA = 0xe;

	/** LOC_ALPHA_FUNC      EQU   15 */
	public static final int LOC_ALPHA_FUNC = 0xf;

	/** LOC_ALPHA_EVAL      EQU   16 */
	public static final int LOC_ALPHA_EVAL = 0x10;

	/** LOC_OBJECT_DATA     EQU   17 */
	public static final int LOC_OBJECT_DATA = 0x11;

	/** LOC_LOCAL_STORAGE   EQU   19 */
	public static final int LOC_LOCAL_STORAGE = 0x13;

	/** LOC_FACTORY_DATA    EQU   20 */
	public static final int LOC_FACTORY_DATA = 0x14;

	/** LOC_XML_TEXT        EQU   21 */
	public static final int LOC_XML_TEXT = 0x15;

	/**  <br/>
	    DATE_FORMAT         DS    FL1 */
	protected static BinaryAsIntField DATE_FORMAT = factory.getBinaryAsIntField(1, true);

	/** DATE_FORMAT_YY      EQU   1 */
	public static final int DATE_FORMAT_YY = 0x1;

	/** DATE_FORMAT_YYXX    EQU   2 */
	public static final int DATE_FORMAT_YYXX = 0x2;

	/** DATE_FORMAT_YYXXXX  EQU   3 */
	public static final int DATE_FORMAT_YYXXXX = 0x3;

	/** DATE_FORMAT_YYXXX   EQU   4 */
	public static final int DATE_FORMAT_YYXXX = 0x4;

	/** DATE_FORMAT_YYYY    EQU   5 */
	public static final int DATE_FORMAT_YYYY = 0x5;

	/** DATE_FORMAT_YYYYXX  EQU   6 */
	public static final int DATE_FORMAT_YYYYXX = 0x6;

	/** DATE_FORMAT_YYYYXXXX EQU  7 */
	public static final int DATE_FORMAT_YYYYXXXX = 0x7;

	/** DATE_FORMAT_YYYYXXX  EQU  8 */
	public static final int DATE_FORMAT_YYYYXXX = 0x8;

	/** DATE_FORMAT_YYX     EQU   9 */
	public static final int DATE_FORMAT_YYX = 0x9;

	/** DATE_FORMAT_YYYYX   EQU   10 */
	public static final int DATE_FORMAT_YYYYX = 0xa;

	/** DATE_FORMAT_XXYY    EQU   22 */
	public static final int DATE_FORMAT_XXYY = 0x16;

	/** DATE_FORMAT_XXXXYY  EQU   23 */
	public static final int DATE_FORMAT_XXXXYY = 0x17;

	/** DATE_FORMAT_XXXYY   EQU   24 */
	public static final int DATE_FORMAT_XXXYY = 0x18;

	/** DATE_FORMAT_XXYYYY  EQU   26 */
	public static final int DATE_FORMAT_XXYYYY = 0x1a;

	/** DATE_FORMAT_XXXXYYYY EQU  27 */
	public static final int DATE_FORMAT_XXXXYYYY = 0x1b;

	/** DATE_FORMAT_XXXYYYY EQU   28 */
	public static final int DATE_FORMAT_XXXYYYY = 0x1c;

	/** DATE_FORMAT_XYY     EQU   29 */
	public static final int DATE_FORMAT_XYY = 0x1d;

	/** DATE_FORMAT_XYYYY   EQU   30 */
	public static final int DATE_FORMAT_XYYYY = 0x1e;

	/**  <br/>
	    FLAGS4              DS    XL1 */
	protected static BinaryAsIntField FLAGS4 = factory.getBinaryAsIntField(1, false);

	/** * For symbols that have attribute Numeric: <br/>
	    FLG4_NUMERIC_NATIONAL EQU X'80' */
	public static final int FLG4_NUMERIC_NATIONAL = 0x80;

	/** * For symbols that have attribute Elementary char: <br/>
	    FLG4_NATIONAL       EQU   X'80' */
	public static final int FLG4_NATIONAL = 0x80;

	/** FLG4_NATIONAL_EDITED EQU  X'40' */
	public static final int FLG4_NATIONAL_EDITED = 0x40;

	/** * For Symbols that have attribute Group: <br/>
	    FLG4_GROUP_USAGE_NATIONAL EQU X'80' */
	public static final int FLG4_GROUP_USAGE_NATIONAL = 0x80;

	/**  <br/>
	                        DS    FL3   Reserved */
	static { factory.getBinaryAsIntField(3, true); }

	/** ADDR_INFO           DS    F     For host, the base locater number      x <br/>
	                                    and displacement. */
	protected static BinaryAsIntField ADDR_INFO = factory.getBinaryAsIntField(4, true);

	/** ADDR_BL_MASK        EQU   X'0FFFF000'  Base Locator number */
	public static final int ADDR_BL_MASK = 0xffff000;

	/** ADDR_DISP_MASK      EQU   X'00000FFF'  Displacement off Base locator */
	public static final int ADDR_DISP_MASK = 0xfff;

	/**  <br/>
	    STRUCT_OFFSET       DS    AL4   Offset of symbol within structure.     x <br/>
	                                    Zero for variably located items. */
	protected static BinaryAsIntField STRUCT_OFFSET = factory.getBinaryAsIntField(4, true);

	/** PARENT_OFFSET       DS    AL4   Offset of symbol from immediate parent */
	protected static BinaryAsIntField PARENT_OFFSET = factory.getBinaryAsIntField(4, true);

	/** PARENT_ID           DS    F     Symbol ID of immediate parent */
	protected static BinaryAsIntField PARENT_ID = factory.getBinaryAsIntField(4, true);

	/** REDEFINED_ID        DS    F     ID of symbold that this item redefs */
	protected static BinaryAsIntField REDEFINED_ID = factory.getBinaryAsIntField(4, true);

	/** STRT_RENAMED_ID     DS    F     If this is L66, the id of the starting x <br/>
	                                    symbol that this item renames, else 0. */
	protected static BinaryAsIntField STRT_RENAMED_ID = factory.getBinaryAsIntField(4, true);

	/** END_RENAMED_ID      DS    F */
	protected static BinaryAsIntField END_RENAMED_ID = factory.getBinaryAsIntField(4, true);

	/** PROGNM_SYMBOL_ID    DS    F */
	protected static BinaryAsIntField PROGNM_SYMBOL_ID = factory.getBinaryAsIntField(4, true);

	/** OCCURS_MINIMUM      DS    F */
	protected static BinaryAsIntField OCCURS_MINIMUM = factory.getBinaryAsIntField(4, true);

	/**                     ORG   OCCURS_MINIMUM <br/>
	    PARAGRAPH_ID        DS    F */
	static { factory.incrementOffset(-4); }
	protected static BinaryAsIntField PARAGRAPH_ID = factory.getBinaryAsIntField(4, true);

	/** OCCURS_MAXIMUM      DS    F */
	protected static BinaryAsIntField OCCURS_MAXIMUM = factory.getBinaryAsIntField(4, true);

	/**                     ORG   OCCURS_MAXIMUM <br/>
	    SECTION_ID          DS    F */
	static { factory.incrementOffset(-4); }
	protected static BinaryAsIntField SECTION_ID = factory.getBinaryAsIntField(4, true);

	/** DIMENSIONS          DS    F     Number of dimensions */
	protected static BinaryAsIntField DIMENSIONS = factory.getBinaryAsIntField(4, true);

	/**                     DS    CL12  Reserved */
	static { factory.getStringField(12, true); }

	/** VALUE_PAIRS_COUNT   DS    H */
	protected static BinaryAsIntField VALUE_PAIRS_COUNT = factory.getBinaryAsIntField(2, true);

	/** SYMBOL_NAME_LEN     DS    H */
	protected static BinaryAsIntField SYMBOL_NAME_LEN = factory.getBinaryAsIntField(2, true);

	/** PICTURE_DATA_LEN    DS    H */
	protected static BinaryAsIntField PICTURE_DATA_LEN = factory.getBinaryAsIntField(2, true);

	/** INITIAL_VAL_LEN     DS    H */
	protected static BinaryAsIntField INITIAL_VAL_LEN = factory.getBinaryAsIntField(2, true);

	/** ODO_SYMBOL_ID       DS    F */
	protected static BinaryAsIntField ODO_SYMBOL_ID = factory.getBinaryAsIntField(4, true);

	/** KEYS_COUNT          DS    H */
	protected static BinaryAsIntField KEYS_COUNT = factory.getBinaryAsIntField(2, true);

	/** INDEX_COUNT         DS    H */
	protected static BinaryAsIntField INDEX_COUNT = factory.getBinaryAsIntField(2, true);

	/** SYMBOL_NAME         DS    CL0   Length varies by SYMBOL_NAME_LEN */
	protected static StringField SYMBOL_NAME = factory.getStringField(0, false);

	/** PICTURE_DATA        DS    CL0   Length varies by PICTURE_DATA_LEN */
	protected static StringField PICTURE_DATA = factory.getStringField(0, false);
	
	/** INDEX_ID_LIST       DS    (INDEX_COUNT)F  */
	/** KEYS                DS    (KEYS_COUNT)XL8 */
	
	/** VALUE_CLAUSE        DS    CL0  Length varies by VALUE_LEN */
	protected static StringField VALUE_CLAUSE = factory.getStringField(0, false);
	
	// Instance variables used to cache field values 
	private Integer symbolId;
	private Integer lineNum;
	private Integer level;
	private Integer qualInd;
	private Integer symbolType;
	private Integer symbolAttr;
	private Integer clauses;
	private Integer flags1;
	private Integer flags2;
	private Integer flags3;
	private Integer fileOrg;
	private Integer usageClause;
	private Integer signClause;
	private Integer indicators;
	private Integer size;
	private Integer precision;
	private Integer scale;
	private Integer baseLocType;
	private Integer dateFormat;
	private Integer flags4;
	private Integer addrInfo;
	private Integer structOffset;
	private Integer parentOffset;
	private Integer parentId;
	private Integer redefinedId;
	private Integer strtRenamedId;
	private Integer endRenamedId;
	private Integer prognmSymbolId;
	private Integer occursMinimum;
	private Integer paragraphId;
	private Integer occursMaximum;
	private Integer sectionId;
	private Integer dimensions;
	private Integer valuePairsCount;
	private Integer symbolNameLen;
	private Integer pictureDataLen;
	private Integer initialValLen;
	private Integer odoSymbolId;
	private Integer keysCount;
	private Integer indexCount;
	private String  symbolName;
	private String	pictureData;
	private String  valueClause;


	/**
	 * Construct an instance on an existing byte array.
	 * @param buffer the byte array used as backing memory for the structure
	 */
	public ADataSymbolRecord(byte[] buffer) {
		super(buffer);
		if (buffer.length < SYMBOL_NAME.getOffset()) {
			throw new IllegalArgumentException("buffer length");
		}
	}	


	public int getSymbolId() {
		if (symbolId == null) {
			symbolId = new Integer(SYMBOL_ID.getInt(bytes));
		}
		return symbolId.intValue();
	}

	public void setSymbolId(int symbolId) {
		if (SYMBOL_ID.equals(this.symbolId, symbolId))
			return;
		SYMBOL_ID.putInt(symbolId, bytes);
		this.symbolId = new Integer(symbolId);
	}

	public int getLineNum() {
		if (lineNum == null) {
			lineNum = new Integer(LINE_NUM.getInt(bytes));
		}
		return lineNum.intValue();
	}

	public void setLineNum(int lineNum) {
		if (LINE_NUM.equals(this.lineNum, lineNum))
			return;
		LINE_NUM.putInt(lineNum, bytes);
		this.lineNum = new Integer(lineNum);
	}

	public int getLevel() {
		if (level == null) {
			level = new Integer(LEVEL.getInt(bytes));
		}
		return level.intValue();
	}

	public void setLevel(int level) {
		if (LEVEL.equals(this.level, level))
			return;
		LEVEL.putInt(level, bytes);
		this.level = new Integer(level);
	}

	public int getQualInd() {
		if (qualInd == null) {
			qualInd = new Integer(QUAL_IND.getInt(bytes));
		}
		return qualInd.intValue();
	}

	public void setQualInd(int qualInd) {
		if (QUAL_IND.equals(this.qualInd, qualInd))
			return;
		QUAL_IND.putInt(qualInd, bytes);
		this.qualInd = new Integer(qualInd);
	}

	public int getSymbolType() {
		if (symbolType == null) {
			symbolType = new Integer(SYMBOL_TYPE.getInt(bytes));
		}
		return symbolType.intValue();
	}

	public void setSymbolType(int symbolType) {
		if (SYMBOL_TYPE.equals(this.symbolType, symbolType))
			return;
		SYMBOL_TYPE.putInt(symbolType, bytes);
		this.symbolType = new Integer(symbolType);
	}

	public int getSymbolAttr() {
		if (symbolAttr == null) {
			symbolAttr = new Integer(SYMBOL_ATTR.getInt(bytes));
		}
		return symbolAttr.intValue();
	}

	public void setSymbolAttr(int symbolAttr) {
		if (SYMBOL_ATTR.equals(this.symbolAttr, symbolAttr))
			return;
		SYMBOL_ATTR.putInt(symbolAttr, bytes);
		this.symbolAttr = new Integer(symbolAttr);
	}

	public int getClauses() {
		if (clauses == null) {
			clauses = new Integer(CLAUSES.getInt(bytes));
		}
		return clauses.intValue();
	}

	public void setClauses(int clauses) {
		if (CLAUSES.equals(this.clauses, clauses))
			return;
		CLAUSES.putInt(clauses, bytes);
		this.clauses = new Integer(clauses);
	}

	public int getFlags1() {
		if (flags1 == null) {
			flags1 = new Integer(FLAGS1.getInt(bytes));
		}
		return flags1.intValue();
	}

	public void setFlags1(int flags1) {
		if (FLAGS1.equals(this.flags1, flags1))
			return;
		FLAGS1.putInt(flags1, bytes);
		this.flags1 = new Integer(flags1);
	}

	public int getFlags2() {
		if (flags2 == null) {
			flags2 = new Integer(FLAGS2.getInt(bytes));
		}
		return flags2.intValue();
	}

	public void setFlags2(int flags2) {
		if (FLAGS2.equals(this.flags2, flags2))
			return;
		FLAGS2.putInt(flags2, bytes);
		this.flags2 = new Integer(flags2);
	}

	public int getFlags3() {
		if (flags3 == null) {
			flags3 = new Integer(FLAGS3.getInt(bytes));
		}
		return flags3.intValue();
	}

	public void setFlags3(int flags3) {
		if (FLAGS3.equals(this.flags3, flags3))
			return;
		FLAGS3.putInt(flags3, bytes);
		this.flags3 = new Integer(flags3);
	}

	public int getFileOrg() {
		if (fileOrg == null) {
			fileOrg = new Integer(FILE_ORG.getInt(bytes));
		}
		return fileOrg.intValue();
	}

	public void setFileOrg(int fileOrg) {
		if (FILE_ORG.equals(this.fileOrg, fileOrg))
			return;
		FILE_ORG.putInt(fileOrg, bytes);
		this.fileOrg = new Integer(fileOrg);
	}

	public int getUsageClause() {
		if (usageClause == null) {
			usageClause = new Integer(USAGE_CLAUSE.getInt(bytes));
		}
		return usageClause.intValue();
	}

	public void setUsageClause(int usageClause) {
		if (USAGE_CLAUSE.equals(this.usageClause, usageClause))
			return;
		USAGE_CLAUSE.putInt(usageClause, bytes);
		this.usageClause = new Integer(usageClause);
	}

	public int getSignClause() {
		if (signClause == null) {
			signClause = new Integer(SIGN_CLAUSE.getInt(bytes));
		}
		return signClause.intValue();
	}

	public void setSignClause(int signClause) {
		if (SIGN_CLAUSE.equals(this.signClause, signClause))
			return;
		SIGN_CLAUSE.putInt(signClause, bytes);
		this.signClause = new Integer(signClause);
	}

	public int getIndicators() {
		if (indicators == null) {
			indicators = new Integer(INDICATORS.getInt(bytes));
		}
		return indicators.intValue();
	}

	public void setIndicators(int indicators) {
		if (INDICATORS.equals(this.indicators, indicators))
			return;
		INDICATORS.putInt(indicators, bytes);
		this.indicators = new Integer(indicators);
	}

	public int getSize() {
		if (size == null) {
			size = new Integer(SIZE.getInt(bytes));
		}
		return size.intValue();
	}

	public void setSize(int size) {
		if (SIZE.equals(this.size, size))
			return;
		SIZE.putInt(size, bytes);
		this.size = new Integer(size);
	}

	public int getPrecision() {
		if (precision == null) {
			precision = new Integer(PRECISION.getInt(bytes));
		}
		return precision.intValue();
	}

	public void setPrecision(int precision) {
		if (PRECISION.equals(this.precision, precision))
			return;
		PRECISION.putInt(precision, bytes);
		this.precision = new Integer(precision);
	}

	public int getScale() {
		if (scale == null) {
			scale = new Integer(SCALE.getInt(bytes));
		}
		return scale.intValue();
	}

	public void setScale(int scale) {
		if (SCALE.equals(this.scale, scale))
			return;
		SCALE.putInt(scale, bytes);
		this.scale = new Integer(scale);
	}

	public int getBaseLocType() {
		if (baseLocType == null) {
			baseLocType = new Integer(BASE_LOC_TYPE.getInt(bytes));
		}
		return baseLocType.intValue();
	}

	public void setBaseLocType(int baseLocType) {
		if (BASE_LOC_TYPE.equals(this.baseLocType, baseLocType))
			return;
		BASE_LOC_TYPE.putInt(baseLocType, bytes);
		this.baseLocType = new Integer(baseLocType);
	}

	public int getDateFormat() {
		if (dateFormat == null) {
			dateFormat = new Integer(DATE_FORMAT.getInt(bytes));
		}
		return dateFormat.intValue();
	}

	public void setDateFormat(int dateFormat) {
		if (DATE_FORMAT.equals(this.dateFormat, dateFormat))
			return;
		DATE_FORMAT.putInt(dateFormat, bytes);
		this.dateFormat = new Integer(dateFormat);
	}

	public int getFlags4() {
		if (flags4 == null) {
			flags4 = new Integer(FLAGS4.getInt(bytes));
		}
		return flags4.intValue();
	}

	public void setFlags4(int flags4) {
		if (FLAGS4.equals(this.flags4, flags4))
			return;
		FLAGS4.putInt(flags4, bytes);
		this.flags4 = new Integer(flags4);
	}

	public int getAddrInfo() {
		if (addrInfo == null) {
			addrInfo = new Integer(ADDR_INFO.getInt(bytes));
		}
		return addrInfo.intValue();
	}

	public void setAddrInfo(int addrInfo) {
		if (ADDR_INFO.equals(this.addrInfo, addrInfo))
			return;
		ADDR_INFO.putInt(addrInfo, bytes);
		this.addrInfo = new Integer(addrInfo);
	}

	public int getStructOffset() {
		if (structOffset == null) {
			structOffset = new Integer(STRUCT_OFFSET.getInt(bytes));
		}
		return structOffset.intValue();
	}

	public void setStructOffset(int structOffset) {
		if (STRUCT_OFFSET.equals(this.structOffset, structOffset))
			return;
		STRUCT_OFFSET.putInt(structOffset, bytes);
		this.structOffset = new Integer(structOffset);
	}

	public int getParentOffset() {
		if (parentOffset == null) {
			parentOffset = new Integer(PARENT_OFFSET.getInt(bytes));
		}
		return parentOffset.intValue();
	}

	public void setParentOffset(int parentOffset) {
		if (PARENT_OFFSET.equals(this.parentOffset, parentOffset))
			return;
		PARENT_OFFSET.putInt(parentOffset, bytes);
		this.parentOffset = new Integer(parentOffset);
	}

	public int getParentId() {
		if (parentId == null) {
			parentId = new Integer(PARENT_ID.getInt(bytes));
		}
		return parentId.intValue();
	}

	public void setParentId(int parentId) {
		if (PARENT_ID.equals(this.parentId, parentId))
			return;
		PARENT_ID.putInt(parentId, bytes);
		this.parentId = new Integer(parentId);
	}

	public int getRedefinedId() {
		if (redefinedId == null) {
			redefinedId = new Integer(REDEFINED_ID.getInt(bytes));
		}
		return redefinedId.intValue();
	}

	public void setRedefinedId(int redefinedId) {
		if (REDEFINED_ID.equals(this.redefinedId, redefinedId))
			return;
		REDEFINED_ID.putInt(redefinedId, bytes);
		this.redefinedId = new Integer(redefinedId);
	}

	public int getStrtRenamedId() {
		if (strtRenamedId == null) {
			strtRenamedId = new Integer(STRT_RENAMED_ID.getInt(bytes));
		}
		return strtRenamedId.intValue();
	}

	public void setStrtRenamedId(int strtRenamedId) {
		if (STRT_RENAMED_ID.equals(this.strtRenamedId, strtRenamedId))
			return;
		STRT_RENAMED_ID.putInt(strtRenamedId, bytes);
		this.strtRenamedId = new Integer(strtRenamedId);
	}

	public int getEndRenamedId() {
		if (endRenamedId == null) {
			endRenamedId = new Integer(END_RENAMED_ID.getInt(bytes));
		}
		return endRenamedId.intValue();
	}

	public void setEndRenamedId(int endRenamedId) {
		if (END_RENAMED_ID.equals(this.endRenamedId, endRenamedId))
			return;
		END_RENAMED_ID.putInt(endRenamedId, bytes);
		this.endRenamedId = new Integer(endRenamedId);
	}

	public int getPrognmSymbolId() {
		if (prognmSymbolId == null) {
			prognmSymbolId = new Integer(PROGNM_SYMBOL_ID.getInt(bytes));
		}
		return prognmSymbolId.intValue();
	}

	public void setPrognmSymbolId(int prognmSymbolId) {
		if (PROGNM_SYMBOL_ID.equals(this.prognmSymbolId, prognmSymbolId))
			return;
		PROGNM_SYMBOL_ID.putInt(prognmSymbolId, bytes);
		this.prognmSymbolId = new Integer(prognmSymbolId);
	}

	public int getOccursMinimum() {
		if (occursMinimum == null) {
			occursMinimum = new Integer(OCCURS_MINIMUM.getInt(bytes));
		}
		return occursMinimum.intValue();
	}

	public void setOccursMinimum(int occursMinimum) {
		if (OCCURS_MINIMUM.equals(this.occursMinimum, occursMinimum))
			return;
		OCCURS_MINIMUM.putInt(occursMinimum, bytes);
		this.occursMinimum = new Integer(occursMinimum);
	}

	public int getParagraphId() {
		if (paragraphId == null) {
			paragraphId = new Integer(PARAGRAPH_ID.getInt(bytes));
		}
		return paragraphId.intValue();
	}

	public void setParagraphId(int paragraphId) {
		if (PARAGRAPH_ID.equals(this.paragraphId, paragraphId))
			return;
		PARAGRAPH_ID.putInt(paragraphId, bytes);
		this.paragraphId = new Integer(paragraphId);
	}

	public int getOccursMaximum() {
		if (occursMaximum == null) {
			occursMaximum = new Integer(OCCURS_MAXIMUM.getInt(bytes));
		}
		return occursMaximum.intValue();
	}

	public void setOccursMaximum(int occursMaximum) {
		if (OCCURS_MAXIMUM.equals(this.occursMaximum, occursMaximum))
			return;
		OCCURS_MAXIMUM.putInt(occursMaximum, bytes);
		this.occursMaximum = new Integer(occursMaximum);
	}

	public int getSectionId() {
		if (sectionId == null) {
			sectionId = new Integer(SECTION_ID.getInt(bytes));
		}
		return sectionId.intValue();
	}

	public void setSectionId(int sectionId) {
		if (SECTION_ID.equals(this.sectionId, sectionId))
			return;
		SECTION_ID.putInt(sectionId, bytes);
		this.sectionId = new Integer(sectionId);
	}

	public int getDimensions() {
		if (dimensions == null) {
			dimensions = new Integer(DIMENSIONS.getInt(bytes));
		}
		return dimensions.intValue();
	}

	public void setDimensions(int dimensions) {
		if (DIMENSIONS.equals(this.dimensions, dimensions))
			return;
		DIMENSIONS.putInt(dimensions, bytes);
		this.dimensions = new Integer(dimensions);
	}

	public int getValuePairsCount() {
		if (valuePairsCount == null) {
			valuePairsCount = new Integer(VALUE_PAIRS_COUNT.getInt(bytes));
		}
		return valuePairsCount.intValue();
	}

	public void setValuePairsCount(int valuePairsCount) {
		if (VALUE_PAIRS_COUNT.equals(this.valuePairsCount, valuePairsCount))
			return;
		VALUE_PAIRS_COUNT.putInt(valuePairsCount, bytes);
		this.valuePairsCount = new Integer(valuePairsCount);
	}

	public int getSymbolNameLen() {
		if (symbolNameLen == null) {
			symbolNameLen = new Integer(SYMBOL_NAME_LEN.getInt(bytes));
		}
		return symbolNameLen.intValue();
	}

	public void setSymbolNameLen(int symbolNameLen) {
		if (SYMBOL_NAME_LEN.equals(this.symbolNameLen, symbolNameLen))
			return;
		SYMBOL_NAME_LEN.putInt(symbolNameLen, bytes);
		this.symbolNameLen = new Integer(symbolNameLen);
	}

	public int getPictureDataLen() {
		if (pictureDataLen == null) {
			pictureDataLen = new Integer(PICTURE_DATA_LEN.getInt(bytes));
		}
		return pictureDataLen.intValue();
	}

	public void setPictureDataLen(int pictureDataLen) {
		if (PICTURE_DATA_LEN.equals(this.pictureDataLen, pictureDataLen))
			return;
		PICTURE_DATA_LEN.putInt(pictureDataLen, bytes);
		this.pictureDataLen = new Integer(pictureDataLen);
	}

	public int getInitialValLen() {
		if (initialValLen == null) {
			initialValLen = new Integer(INITIAL_VAL_LEN.getInt(bytes));
		}
		return initialValLen.intValue();
	}

	public void setInitialValLen(int initialValLen) {
		if (INITIAL_VAL_LEN.equals(this.initialValLen, initialValLen))
			return;
		INITIAL_VAL_LEN.putInt(initialValLen, bytes);
		this.initialValLen = new Integer(initialValLen);
	}

	public int getOdoSymbolId() {
		if (odoSymbolId == null) {
			odoSymbolId = new Integer(ODO_SYMBOL_ID.getInt(bytes));
		}
		return odoSymbolId.intValue();
	}

	public void setOdoSymbolId(int odoSymbolId) {
		if (ODO_SYMBOL_ID.equals(this.odoSymbolId, odoSymbolId))
			return;
		ODO_SYMBOL_ID.putInt(odoSymbolId, bytes);
		this.odoSymbolId = new Integer(odoSymbolId);
	}

	public int getKeysCount() {
		if (keysCount == null) {
			keysCount = new Integer(KEYS_COUNT.getInt(bytes));
		}
		return keysCount.intValue();
	}

	public void setKeysCount(int keysCount) {
		if (KEYS_COUNT.equals(this.keysCount, keysCount))
			return;
		KEYS_COUNT.putInt(keysCount, bytes);
		this.keysCount = new Integer(keysCount);
	}

	public int getIndexCount() {
		if (indexCount == null) {
			indexCount = new Integer(INDEX_COUNT.getInt(bytes));
		}
		return indexCount.intValue();
	}

	public void setIndexCount(int indexCount) {
		if (INDEX_COUNT.equals(this.indexCount, indexCount))
			return;
		INDEX_COUNT.putInt(indexCount, bytes);
		this.indexCount = new Integer(indexCount);
	}
	
	public String getSymbolName() {
		if (this.symbolName == null) {
			StringField field = new StringField(
										SYMBOL_NAME.getOffset(), 
										getSymbolNameLen(), 
										true);
			this.symbolName = field.getString(bytes);
		}
		return this.symbolName;
	}
	
	public String getPictureData() {
		if (this.pictureData == null) {
			// The offset to the end of the symbol name
			int offset = SYMBOL_NAME.getOffset() + getSymbolNameLen();
			StringField field = new StringField(offset, 
												getPictureDataLen(),
												true);
			this.pictureData = field.getString(bytes);
		}
		return this.pictureData;
	}

	/**
	 * Answers the first value clause from the record, or null
	 * if there are no clauses.
	 */
	public String getValueClause() {
		int len = getInitialValLen();
		if (len== 0) {
			return null;
		}
		if (valueClause == null) {
			// calculate the offset to the first value pair, based on
			// the presence of other intervening fields.
			int offset = VALUE_CLAUSE.getOffset()
						+ getSymbolNameLen()
						+ getPictureDataLen()
						+ getIndexCount() * 4
						+ getKeysCount() * 8;
			BinaryAsIntField lenField = new BinaryAsIntField(offset, 2, true);
			len = lenField.getInt(bytes);
			StringField field = new StringField(offset+2, len, true);
			valueClause = field.getString(bytes);
		}
		return valueClause;
	}
	
	public boolean isCondition() {
		return getSymbolAttr() == SYMA_CONDITION;	
	}

	public boolean isGroup() {
		return getSymbolAttr() == SYMA_GROUP;	
	}

	public boolean isField() {
		int attr = getSymbolAttr();
		return (attr == ADataSymbolRecord.SYMA_NUMERIC ||
				   attr == ADataSymbolRecord.SYMA_CHAR ||
				   attr == ADataSymbolRecord.SYMA_POINTER);	
	}

	public boolean isNumeric() {
		return getSymbolAttr() == SYMA_NUMERIC;
	}

	public boolean isCharacter() {
		return getSymbolAttr() == SYMA_CHAR;
	}

	public boolean isSigned() {
		return getSignClause() > 0 || getPictureData().indexOf('S') != -1;
	}
	
	public boolean isRedefinesClause() {
		return (getClauses() & CLAUSE_REDEFINES) != 0;
	}
	
	public boolean isOccursClause() {
		return (getClauses() & CLAUSE_OCCURS) != 0;
	}
	
	public boolean isOccursDependingOnClause() {
		return (getClauses() & CLAUSE_ODO) != 0;
	}
	
	/**
	 * Return true if the symbol represents a COMP-5 field.  This is not explicitly covered
	 * in the USAGE bit field, so check the SCALED_NEG bit on, which is set for COMP-5 items.
	 * If the scale is also 0, we assume COMP-5.
	 */
	public boolean isComp5() {
		int comp5 = FLG2_BINARY + FLG2_SCALED_NEG;
		return getScale() == 0 && (getFlags2() & comp5) == comp5;
	}
	
	public boolean isScaledNegative() {
		return (getFlags2() & FLG2_SCALED_NEG) != 0;
	}
	
	public boolean isBinary() {
		return (getFlags2() & FLG2_BINARY) != 0;
	}
	
	public boolean isExternalFloatingPoint() {
		return (getFlags2() & FLG2_EXT_FLOAT) != 0;
	}

	public boolean isNumericEdited() {
		return (getFlags2() & FLG2_NUMERIC_EDIT) != 0;
	}

	public boolean isInternalFloatingPoint() {
		return (getFlags2() & FLG2_INT_FLOAT) != 0;
	}
	
	public boolean isPackedDecimal() {
		return (getFlags2() & FLG2_PACKED) != 0;
	}
	
	public boolean isExternalDecimal() {
		return (getFlags2() & FLG2_EXT_DECIMAL) != 0;
	}
	
	public boolean isUsageNationalOrDBCS() {
		int u = getUsageClause();
		return u == USAGE_NATIONAL || u == USAGE_DISPLAY1;
	}
	
	public boolean isNumericNational() {
		return (getFlags4() & (FLG4_NUMERIC_NATIONAL | FLG4_NATIONAL_EDITED)) != 0;
	}
	
	public boolean isShowMantissaPlusSign() {
		return getSignClause() == 0x1;
	}
	
	public boolean hasNumericPictureClause() {
		String p = getPictureData();
		if (p == null || p.length() == 0) return false;
		for (int i=0; i<p.length(); i++) {
			if ("9PSV".indexOf(p.charAt(i)) < 0) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Return true if plus signs are to be used to represent positive exponent
	 * values.  The symbol sign clause is not sufficient to determine this, so
	 * check for the existence of a minus sign in the picture clause.
	 * <p>The following table shows the sign clause values for mantissa and
	 * exponent sign indicators as produced by the COBOL compiler.
	 * <dl>
	 * <dt>PIC +9(4).99E+99</dt><dd>SIGN = 0x1</dd>
	 * <dt>PIC -9(4).99E-99</dt><dd>SIGN = 0x0</dd>
	 * <dt>PIC +9(4).99E-99</dt><dd>SIGN = 0x1</dd>
	 * <dt>PIC -9(4).99E-99</dt><dd>SIGN = 0x4</dd>
	 * </dl>
	 */
	public boolean isShowExponentPlusSign() {
		return getSignClause() == 0x4 || (getPictureData().indexOf('-') == -1);
	}

	public boolean isImpliedDecimal() {
		return getPictureData().indexOf('V') != -1;
	}
	
	public boolean isBlankWhenZero() {
		return (getIndicators() & IND_BLANK_WHEN_ZERO) != 0;
	}
	
	public boolean isJustified() {
		return (getIndicators() & IND_JUSTIFIED) != 0;
	}
	
	public boolean isSignTrailing() {
		return getSignClause() == SIGN_TRAIL || getSignClause() == SIGN_TRAIL_SEP;
	}
	
	public boolean isSignSeparate() {
		return getSignClause() == SIGN_LEADING_SEP || getSignClause() == SIGN_TRAIL_SEP;
	}
	
	public boolean isVarLoc() {
		return getBaseLocType() == LOC_VARLOC_ITEMS;
	}
	
	/**
	 * Write debugging information for the receiver.
	 */
	public void dumpOn(Writer writer) throws IOException {
		// super.dumpOn(writer);
		writer.write("Symbol: ");
		writer.write(getSymbolName());
		writer.write("  ID=");
		writer.write(Integer.toString(getSymbolId()));
		writer.write("  Line=");
		writer.write(Integer.toString(getLineNum()));
		writer.write("  Level=");
		writer.write(Integer.toString(getLevel()));
		writer.write("  QualInd=");
		writer.write(Integer.toString(getQualInd()));
		writer.write("\n");
		writer.write("  Type=0x");
		writer.write(ByteUtil.toHexString(getSymbolType(),2));
		writer.write("  Attr=0x");
		writer.write(ByteUtil.toHexString(getSymbolAttr(),2));
		writer.write("  Clauses=0x");
		writer.write(ByteUtil.toHexString(getClauses(),2));
		writer.write("\n");
		writer.write("  Flags1=0x");
		writer.write(ByteUtil.toHexString(getFlags1(),2));
		writer.write("  Flags2=0x");
		writer.write(ByteUtil.toHexString(getFlags2(),2));
		writer.write("  Flags3=0x");
		writer.write(ByteUtil.toHexString(getFlags3(),2));
		writer.write("  Flags4=0x");
		writer.write(ByteUtil.toHexString(getFlags4(),2));
		writer.write("\n");
		writer.write("  Usage=0x");
		writer.write(ByteUtil.toHexString(getUsageClause(),2));
		writer.write("  Sign=0x");
		writer.write(ByteUtil.toHexString(getSignClause(),2));
		writer.write("  Indicators=0x");
		writer.write(ByteUtil.toHexString(getIndicators(),2));
		writer.write("\n");
		writer.write("  Size=");
		writer.write(Integer.toString(getSize()));
		writer.write("  Precision=");
		writer.write(Integer.toString(getPrecision()));
		writer.write("  Scale=");
		writer.write(Integer.toString(getScale()));
		writer.write("  BaseLocType=");
		writer.write(Integer.toString(getBaseLocType()));
		writer.write("\n");
		writer.write("  AddrInfo=0x");
		writer.write(ByteUtil.toHexString(getAddrInfo(),8));
		writer.write("  StructOffset=0x");
		writer.write(ByteUtil.toHexString(getStructOffset(),8));
		writer.write("  ParentOffset=0x");
		writer.write(ByteUtil.toHexString(getParentOffset(),8));
		writer.write("\n");
		writer.write("  ParentId=");
		writer.write(Integer.toString(getParentId()));
		writer.write("  RedefinedId=");
		writer.write(Integer.toString(getRedefinedId()));
		writer.write("  StrtRenamedId=");
		writer.write(Integer.toString(getStrtRenamedId()));
		writer.write("  EndRenamedId=");
		writer.write(Integer.toString(getEndRenamedId()));
		writer.write("\n");
		writer.write("  OccursMin=");
		writer.write(Integer.toString(getOccursMinimum()));
		writer.write("  OccursMax=");
		writer.write(Integer.toString(getOccursMaximum()));
		writer.write("  Dimensions=");
		writer.write(Integer.toString(getDimensions()));
		writer.write("  ODOId=");
		writer.write(Integer.toString(getOdoSymbolId()));
		writer.write("  IndexCount=");
		writer.write(Integer.toString(getIndexCount()));
		writer.write("\n");		
		writer.write("  PICTURE: ");
		writer.write(getPictureData());
		writer.write("\n");		
		if (getValueClause() != null) {
			writer.write("  VALUE: ");
			writer.write(getValueClause());
			writer.write("\n");
		}
	}
	

}
