/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Specification that represents a COBOL condition (level 88) item.
 */
public class ConditionSpec extends ElementSpec {
	private String valueClause;
	
	static final String XML_ELEMENT_NAME = "condition";

	/**
	 * Construct an instance
	 * @param valueClause
	 */
	public ConditionSpec(String valueClause) {
		this.valueClause = valueClause;
	}

	/**
	 * Return the VALUE clause associated with the condition element
	 */
	public String getValueClause() {
		return valueClause;
	}

	/**
	 * @see #getValueClause()
	 */
	public void setValueClause(String valueClause) {
		this.valueClause = valueClause;
	}
	
	/**
	 * @see ElementSpec#writeXmlOn(XMLStreamWriter, int)
	 * @since 2.4.4
	 */
	@Override
	public void writeXmlOn(XMLStreamWriter writer, int indentLevel)
			throws XMLStreamException {
		writeXmlNewline(writer, indentLevel);
		writer.writeStartElement(XML_ELEMENT_NAME);
		writeXmlAttributesOn(writer);
		writeXmlSourceLinesOn(writer);
		writeXmlNewline(writer, indentLevel);
		writer.writeEndElement();
	}
	
	/**
	 * @see ElementSpec#writeXmlAttributesOn(XMLStreamWriter)
	 * @since 2.4.4
	 */
	@Override
	protected void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		super.writeXmlAttributesOn(writer);
		
		if (getValueClause() != null) {
			writer.writeAttribute("value", getValueClause());
		}
	}
	
	/**
	 * @see ElementSpec#parseXmlFrom(RecordSpecTreeXMLParser)
	 * @since 2.4.4
	 */	
	@Override
	protected void parseXmlFrom(RecordSpecTreeXMLParser parser) throws XMLStreamException {
		super.parseXmlAttributesFrom(parser);
		setValueClause(parser.getAttributeValue("value"));	
		parser.addSpecById(this);
		parseXmlSourceLinesFrom(parser);		
	}
		
}
