/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;


/**
 * An object that represents a RENAMES THRU clause in a COBOL COPYBOOK element.
 */
public class RenameRangeSpec extends RenameSpec {
	private StorageSpec toStorageSpec;
	
	static final String XML_ELEMENT_NAME = "renameRange";
	
	/**
	 * Create an instance.
	 * @param fromStorageSpec
	 * @param toStorageSpec
	 */
	public RenameRangeSpec(StorageSpec fromStorageSpec, StorageSpec toStorageSpec) {
		super(fromStorageSpec);
		this.toStorageSpec = toStorageSpec; 
	}

	/**
	 * The starting element that this element renames.
	 * @return StorageSpec starting storage spec
	 */
	public StorageSpec getToStorageSpec() {
		return toStorageSpec;
	}

	/**
	 * @see #getToStorageSpec()
	 */
	public void setToStorageSpec(StorageSpec toStorageSpec) {
		this.toStorageSpec = toStorageSpec;
	}
	
	/**
	 * A debug representation of the receiver.
	 */
	public String toString() {
		return "Renames From: " + getStorageSpec().getSymbol() + " To: " + getToStorageSpec().getSymbol();
	}

	/**
	 * @see ElementSpec#writeXmlOn(XMLStreamWriter, int)
	 * @since 2.4.4
	 */
	@Override
	public void writeXmlOn(XMLStreamWriter writer, int indentLevel)
			throws XMLStreamException {
		writeXmlNewline(writer, indentLevel);
		writer.writeStartElement(XML_ELEMENT_NAME);
		writeXmlAttributesOn(writer);
		writeXmlSourceLinesOn(writer);
		writeXmlNewline(writer, indentLevel);
		writer.writeEndElement();
	}
	
	/**
	 * @see ElementSpec#writeXmlAttributesOn(XMLStreamWriter)
	 * @since 2.4.4
	 */
	@Override
	protected void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		super.writeXmlAttributesOn(writer);
		writer.writeAttribute("fromStorage", Integer.toString(getStorageSpec().getSymbolId()));
		writer.writeAttribute("toStorage", Integer.toString(getStorageSpec().getSymbolId()));
	}
	
	/**
	 * @see ElementSpec#parseXmlFrom(RecordSpecTreeXMLParser)
	 * @since 2.4.4
	 */
	@Override
	protected void parseXmlFrom(RecordSpecTreeXMLParser parser) throws XMLStreamException {
		super.parseXmlAttributesFrom(parser);
		setStorageSpec(parser.getStorageSpecForId(parser.getAttributeIntValue("fromStorage")));	
		setToStorageSpec(parser.getStorageSpecForId(parser.getAttributeIntValue("toStorage")));	
		parser.addSpecById(this);
		parseXmlSourceLinesFrom(parser);		
	}	
	
}
