/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * An object that represents a RENAMES clause in a COBOL COPYBOOK element.
 */
public class RenameSpec extends ElementSpec {
	private StorageSpec storageSpec;
	
	static final String XML_ELEMENT_NAME = "rename";	
	/**
	 * Create an instance.
	 * @param renamedSpec
	 */
	public RenameSpec(StorageSpec renamedSpec) {
		storageSpec = renamedSpec;
	}

	/**
	 * Return the element that this element renames
	 * @return StorageSpec the renamed spec
	 */
	public StorageSpec getStorageSpec() {
		return storageSpec;
	}

	/**
	 * @see #getStorageSpec()
	 */
	public void setStorageSpec(StorageSpec storageSpec) {
		this.storageSpec = storageSpec;
	}
	
	/**
	 * A debug representation of the receiver.
	 */
	public String toString() {
		return "Renames: " + getStorageSpec().getSymbol();
	}

	@Override
	public void writeXmlOn(XMLStreamWriter writer, int indentLevel)
			throws XMLStreamException {
		writeXmlNewline(writer, indentLevel);
		writer.writeStartElement(XML_ELEMENT_NAME);
		writeXmlAttributesOn(writer);
		writeXmlSourceLinesOn(writer);
		writeXmlNewline(writer, indentLevel);
		writer.writeEndElement();
	}
	
	@Override
	protected void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		super.writeXmlAttributesOn(writer);
		writer.writeAttribute("storage", Integer.toString(getStorageSpec().getSymbolId()));
	}
	
	@Override
	protected void parseXmlFrom(RecordSpecTreeXMLParser parser) throws XMLStreamException {
		super.parseXmlAttributesFrom(parser);
		parser.addSpecById(this);
		setStorageSpec(parser.getStorageSpecForId(parser.getAttributeIntValue("storage")));	
		parseXmlSourceLinesFrom(parser);		
	}	
	
}
