/*
* %Z%%W% %I%
*
===========================================================================
* Licensed Materials - Property of IBM
* "Restricted Materials of IBM"
* (C) Copyright IBM Corp. 2007, 2012. All Rights Reserved
*
===========================================================================
*/
package com.ibm.jzos.recordgen.cobol;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

/**
 * Object that represents the offset for a variably located (varloc) element.
 * All items that follow the introduction of an ODO are variably located.
 */
public class VariableOffset extends Offset {
	private StorageSpec odo;
	
	/**
	 * Create an instance.
	 * @param offset the fixed portion of the offset
	 * @param odo the ODO spec that this offset follows
	 */
	public VariableOffset(int offset, StorageSpec odo) {
		super(offset);
		this.odo = odo; 
	}

	/**
	 * Create an instance.
	 * @param odo the ODO spec that this offset follows
	 */
	public VariableOffset(StorageSpec odo) {
		super(0);
		this.odo = odo; 
	}

	/**
	 * Return the Occurs Depending On spec that this offset is related to.
	 * @return StorageSpec the related ODO
	 */
	public StorageSpec getOdo() {
		return odo;
	}

	/**
	 * @see #getOdo()
	 */
	public void setOdo(StorageSpec odo) {
		this.odo = odo;
	}
	
	/**
	 * A debug representation of the receiver.
	 */
	public String toString() {
		return odo.getOffset() + "->" + odo.getSymbol() + super.toString();
	}
	
	/**
	 * Return a clone of the receiver.
	 */
	public Object clone() {
		return new VariableOffset(offset,odo);
	}
	
	/**
	 * @see ElementSpec#writeXmlAttributesOn(XMLStreamWriter)
	 * @since 2.4.4
	 */
	public void writeXmlAttributesOn(XMLStreamWriter writer) throws XMLStreamException {
		super.writeXmlAttributesOn(writer);
		writer.writeAttribute("offsetDependingOn", 
				Integer.toString(getOdo().getSymbolId()));
	}
}
